@@ -12,17 +12,20 @@ job is to provide a more advanced Table of Contents-style navigation system for
 the ePub beyond what is done in the OPF file.
 """
 
-import openaccess_epub.utils as utils
-import openaccess_epub.utils.element_methods as element_methods
-from openaccess_epub.utils import OrderedSet
-from .publisher_metadata import *
+#Standard Library modules
 from collections import namedtuple
-import os
-import xml.dom.minidom
-from lxml import etree
 import logging
+import os
 
-log = logging.getLogger('NCX')
+#Non-Standard Library modules
+from lxml import etree
+
+#OpenAccess_EPUB modules
+from .publisher_metadata import *
+from openaccess_epub.utils import OrderedSet
+import openaccess_epub.utils.element_methods as element_methods
+
+log = logging.getLogger('openaccess_epub.ncx.ncx')
 
 navpoint = namedtuple('navPoint', 'id, label, playOrder, source, children')
 navtarget = namedtuple('navTarget', 'id, label, source')
@@ -92,7 +95,7 @@ e
         Receives an instance of the Article class. This modifies the internal
         state of the NCX class to focus on the new article for the purposes of
         extracting structural information, and the article authors as metadata.
-        
+
         In Collection Mode, the addition of new articles to the NCX class
         results in cumulative (in order of receipt) content. In Single Input
         Mode, the addition of a new article will erase any information from the
@@ -230,9 +233,12 @@ e
         """
         Sets internal methods to be publisher specific for the article at hand.
         """
-        if self.journal_doi == '10.1371':
+        if self.journal_doi == '10.1371':  # PLoS
             self.get_article_creator = plos_creator
-            self.get_article_title= plos_title
+            self.get_article_title = plos_title
+        elif self.journal_doi == '10.3389':  # Frontiers
+            self.get_article_creator = frontiers_creator
+            self.get_article_title = frontiers_title
         else:
             raise ValueError('This publisher, {0}, is not supported'.format(self.journal_doi))
 
@@ -259,7 +265,7 @@ e
     def reset_metadata(self):
         """
         THe NCX file does not truly exist for metadata, but it has a few
-        elements held over from the Daisy Talking Book specification. The 
+        elements held over from the Daisy Talking Book specification. The
         """
         self.doc_author = OrderedSet()
         self.article_title = ''
