@@ -13,7 +13,7 @@ import os
 import sys
 import logging
 
-log = logging.getLogger('OPSPLoS')
+log = logging.getLogger('openaccess_epub.ops.opsplos')
 
 
 class OPSPLoS(OPSMeta):
@@ -38,9 +38,8 @@ class OPSPLoS(OPSMeta):
         if self.html_tables:
             self.create_tables()
 
-
     def create_main(self):
-        """
+        """Have started to look at this one.
         This method encapsulates the process required to generate the primary
         body of the article. This includes the heading, article info, and main
         text, including some elements of the back matter. It does not include
@@ -69,6 +68,11 @@ class OPSPLoS(OPSMeta):
         #supported elements and add them appropriately to the XML
         self.make_back_matter(body)
 
+        #A work-around for XML with <boxed-text> elements placed irregularly in
+        #the <back> of the article. We cut them over to the back of the body
+        #journal.pgen.0020002 is the known offender
+        self.move_back_boxed_texts(body)
+
         #Handle node conversion
         self.convert_disp_formula_elements(body)
         self.convert_inline_formula_elements(body)
@@ -86,11 +90,7 @@ class OPSPLoS(OPSMeta):
 
         self.convert_graphic_elements(body)
 
-        #TODO: Back matter stuffs
-
         #These come last for a reason
-        self.convert_sec_elements(body)
-        self.convert_div_titles(body)
         self.post_processing_conversion(body)
 
         #Finally, write to a document
@@ -194,6 +194,34 @@ class OPSPLoS(OPSMeta):
 
         self.write_document(os.path.join(self.ops_dir, self.bib_frag[:-4]), self.document)
 
+    def format_nlm_citation(self, nlm_citation):
+        """
+        This method does the job of properly formatting an nlm-citation element
+        for display in the ePub. The nlm-citation element should be passed in
+        as the argument, and this function will return an element containing
+        the formatted data.
+
+        This method relies on the following resources for reference:
+        https://github.com/PLOS/ambra/blob/master/base/src/main/resources/viewnlm-v2.3.xsl
+        https://github.com/PLOS/ambra/blob/master/base/src/main/resources/articleTransform-v3.xsl
+
+        PLoS employs the nlm-citation formatting for mixed-citation and
+        element-citations as well.
+        """
+        #Count the number of authors
+        author_group_count = int(nlm_citation.xpath('count(person-group) + count(collab)'))
+        if author_group_count == 1:
+            pass
+
+    def hyperlink_citation(self, citation):
+        """
+        PLoS adds hyperlinks to locate the resource on the web at the end of
+        its bibliographic citations. This method does that job.
+
+        This method relies on the following resource for reference:
+        https://github.com/PLOS/ambra/blob/master/base/src/main/resources/articleTransform-v3.xsl
+        """
+
     def create_tables(self):
         """
         This method encapsulates the functions necessary to create a file
@@ -231,6 +259,9 @@ class OPSPLoS(OPSMeta):
         self.convert_inline_formula_elements(body)
         self.convert_xref_elements(body)
 
+        #Lastly
+        self.post_processing_conversion(body)
+
         #Finally, write to a document
         self.write_document(os.path.join(self.ops_dir, self.tab_frag[:-4]), self.document)
 
@@ -337,7 +368,7 @@ class OPSPLoS(OPSMeta):
         #Create a list of all those pertaining to the authors
         author_affs = [i for i in affs if 'aff' in i.attrs['id']]
         #Count them, used for formatting
-        author_aff_count = len(self.metadata.front.article_meta.aff)
+        author_aff_count = len(author_affs)
         if author_aff_count > 0:
             affs_list = etree.SubElement(receiving_node, 'ul', {'id': 'affiliations',
                                                                 'class': 'simple'})
@@ -406,15 +437,101 @@ class OPSPLoS(OPSMeta):
         """
         Creates a self citation node for the ArticleInfo of the article.
 
-        This method relies on self.format_self_citation() as an implementation
-        of converting an article's metadata to a plain string, and then adds
-        composes content for the display of that string in the ArticleInfo.
+        This method uses code from this page as a reference implementation:
+        https://github.com/PLOS/ambra/blob/master/base/src/main/resources/articleTransform-v3.xsl
         """
-        citation_text = self.format_self_citation()
         citation_div = etree.SubElement(receiving_el, 'div')
         citation_div.attrib['id'] = 'article-citation'
         b = etree.SubElement(citation_div, 'b')
-        b.text = 'Citation: {0}'.format(citation_text)
+        b.text = 'Citation: '
+
+        #Add author stuff to the citation
+        author_list = self.get_authors_list()
+        for author in author_list:
+            author_index = author_list.index(author)
+            #At the 6th author, simply append an et al., then stop iterating
+            if author_index == 5:
+                element_methods.append_new_text(citation_div, 'et al.', join_str='')
+                break
+            else:
+                #Check if the author contrib has a collab
+                #This will signify unique behavior
+                if len(author.collab) > 0:  #Author element is a collab
+                    #As best as I can tell from PLoS' reference implementation
+                    #The thing to do is append all below, but remove any child
+                    #contrib-group elements
+                    collab = deepcopy(author.collab[0])
+                    for contrib_group in collab.contrib_group:
+                        element_methods.remove(contrib_group)
+                    element_methods.append_all_below(citation_div, collab.node, join_str='')
+                #If the author is not a collab, do this instead
+                else:  # Author element is not a collab
+                    name = author.name[0]
+                    #Note that this does not support eastern names
+                    #Grab the surname information
+                    element_methods.append_new_text(citation_div, name.surname.text, join_str='')
+                    #Make initials from the given-name information
+                    if name.given_names is not None:
+                        #Add a space
+                        element_methods.append_new_text(citation_div, ' ', join_str='')
+                        #Split by whitespace and take first character
+                        given_initials = [i[0] for i in name.given_names.text.split() if i]
+                        for initial in given_initials:
+                            element_methods.append_new_text(citation_div, initial, join_str='')
+                    #If there is a suffix, add its text, but don't include the
+                    #trailing period if there is one
+                    if name.suffix is not None:
+                        #Add a space
+                        element_methods.append_new_text(citation_div, ' ', join_str='')
+                        suffix_text = name.suffix.text
+                        #Check for the trailing period
+                        if suffix_text[-1] == '.':
+                            suffix_text = suffix_text[:-1]
+                        element_methods.append_new_text(citation_div, suffix_text, join_str='')
+                #If this is not the last author to be added, add a ", "
+                #This is satisfied by being less than the 6th author, or less
+                #than the length of the author_list - 1
+                if author_index < 5 or author_index < len(author_list) -1:
+                    element_methods.append_new_text(citation_div, ', ', join_str='')
+        #Add Publication Year to the citation
+        #Find pub-date elements, use pub-type=collection, or else pub-type=ppub
+        for pub_date in self.metadata.front.article_meta.pub_date:
+            #pub_year = '1337'
+            if 'pub-type' not in pub_date.attrs:
+                continue
+            elif pub_date.attrs['pub-type'] == 'collection':
+                pub_year = pub_date.year.text
+                break
+            elif pub_date.attrs['pub-type'] == 'ppub':
+                pub_year = pub_date.year.text
+        element_methods.append_new_text(citation_div, ' ({0}) '.format(pub_year), join_str='')
+        #Add the Article Title to the Citation
+        #As best as I can tell from the reference implementation, they
+        #serialize the article title to text-only, and expunge redundant spaces
+        #This might need later review
+        article_title = self.metadata.front.article_meta.title_group.article_title.node
+        article_title_text = str(etree.tostring(article_title, method='text', encoding='utf-8'), encoding='utf-8')
+        normalized = ' '.join(article_title_text.split())  # Remove redundant whitespace
+        #Add a period unless there is some other valid punctuation
+        if normalized[-1] not in '.?!':
+            normalized += '.'
+        element_methods.append_new_text(citation_div, normalized + ' ', join_str='')
+        #Add the article's journal name using the journal-id of type "nlm-ta"
+        for journal_id in self.metadata.front.journal_meta.journal_id:
+            if 'journal-id-type' not in journal_id.attrs:
+                continue
+            elif journal_id.attrs['journal-id-type'] == 'nlm-ta':
+                journal = journal_id.text
+        element_methods.append_new_text(citation_div, journal + ' ', join_str='')
+        #Add the article's volume, issue, and elocation_id  values
+        volume = self.metadata.front.article_meta.volume.text
+        issue = self.metadata.front.article_meta.issue.text
+        elocation_id = self.metadata.front.article_meta.elocation_id.text
+        stuff = '{0}({1}): {2}. '.format(volume, issue, elocation_id)
+        element_methods.append_new_text(citation_div, stuff, join_str='')
+        #Grab the article's DOI, I'll cheat by using self.doi
+        element_methods.append_new_text(citation_div, 'doi:{0}'.format(self.doi), join_str='')
+        #Done!
 
     def make_article_info_editors(self, editors, receiving_el):
         if not editors:  # No editors
@@ -432,7 +549,7 @@ class OPSPLoS(OPSMeta):
                 first = False
             else:
                 element_methods.append_new_text(editors_div, '; ', join_str='')
-            
+
             if len(editor.anonymous) > 0:
                 element_methods.append_new_text(editors_div, 'Anonymous', join_str='')
             elif len(editor.collab) > 0:
@@ -517,11 +634,11 @@ class OPSPLoS(OPSMeta):
         copyright_string = '\u00A9 '
         if len(permissions.copyright_holder) > 0:
             copyright_string += element_methods.all_text(permissions.copyright_holder[0].node)
-            copyright_string += '. ' 
+            copyright_string += '. '
         if len(permissions.license) > 0:  # I'm assuming only one license
             #Taking only the first license_p element
             license_p = permissions.license[0].license_p[0]
-            #I expect to see only text in the 
+            #I expect to see only text in the
             copyright_string += element_methods.all_text(license_p.node)
         element_methods.append_new_text(copyright_div, copyright_string)
 
@@ -638,7 +755,7 @@ class OPSPLoS(OPSMeta):
         """
         #Back is technically metadata content that needs to be interpreted to
         #presentable content
-        if self.metadata.back is None: 
+        if self.metadata.back is None:
             return
         #The following things are ordered in such a way to adhere to what
         #appears to be a consistent presentation order for PLoS
@@ -671,7 +788,7 @@ class OPSPLoS(OPSMeta):
         #allow it to later be depth-formatted by self.convert_div_titles()
         ack_title = etree.Element('title')
         ack_title.text = 'Acknowledgments'
-        ack.insert(0, ack_title)
+        ack.insert(0, ack_title)  # Make it the first element
         #Append our modified copy of the first ack element to the receiving_el
         receiving_el.append(ack)
 
@@ -744,6 +861,21 @@ class OPSPLoS(OPSMeta):
             notes_sec.attrib['class'] = 'back-notes'
             receiving_el.append(notes_sec)
 
+    def move_back_boxed_texts(self, receiving_el):
+        """
+        The only intended use for this function is to patch a problem seen in
+        at least one PLoS article (journal.pgen.0020002). This will move any
+        <boxed-text> elements over to the receiving element, which is probably
+        the main body.
+        """
+        if self.metadata.back is None:
+            return
+        back_boxed_texts = self.metadata.back.node.findall('.//boxed-text')
+        if len(back_boxed_texts) == 0:
+            return
+        for back_boxed_text in back_boxed_texts:
+            receiving_el.append(back_boxed_text)
+
     def format_date_string(self, date_tuple):
         """
         Receives a date_tuple object, defined in jptsmeta, and outputs a string
@@ -763,16 +895,6 @@ class OPSPLoS(OPSMeta):
                 date_string += ' ' + date_tuple.day.text
             return ', '.join([date_string, date_tuple.year.text])
 
-    def format_self_citation(self):
-        """
-        PLoS articles present a citation for the article itself. This method
-        will return the citation for the article as a string.
-        """
-        #This is not yet fully implemented.
-        #I need clarification/documentation from PLoS
-        #So for now I just put in the DOI
-        return self.doi
-
     def convert_fig_elements(self, top):
         """
         Responsible for the correct conversion of JPTS 3.0 <fig> elements to
@@ -828,7 +950,7 @@ class OPSPLoS(OPSMeta):
         """
         Responsible for the correct conversion of JPTS 3.0 <table-wrap>
         elements to OPS content.
-        
+
         The 'id' attribute is treated as mandatory by this method.
         """
         for table_wrap in top.findall('.//table-wrap'):
@@ -836,7 +958,7 @@ class OPSPLoS(OPSMeta):
             #for child in tab.childNodes:
             #    if child.nodeType == 8:
             #        element_methods.uncomment(child)
-            
+
             #Create a div for all of the table stuff
             table_div = etree.Element('div')
             table_div.attrib['id'] = table_wrap.attrib['id']
@@ -886,7 +1008,7 @@ class OPSPLoS(OPSMeta):
             #
             #If there is a table with no image, then the table should be placed
             #in the text flow.
-            
+
             if graphic is not None:
                 #Create the image path for the graphic
                 xlink_href = element_methods.ns_format(graphic, 'xlink:href')
@@ -1214,6 +1336,18 @@ class OPSPLoS(OPSMeta):
                     suppl_div.append(title)
                 for paragraph in paragraphs:
                     suppl_div.append(paragraph)
+            #This is a fix for odd articles with <p>s outside of <caption>
+            #See journal.pctr.0020006, PLoS themselves fail to format this for
+            #the website, though the .pdf is good
+            #It should be noted that journal.pctr.0020006 does not pass
+            #validation because it places a <p> before a <caption>
+            #By placing this at the end of the method, it conforms to the spec
+            #by expecting such p tags after caption. This causes a hiccup in
+            #the rendering for journal.pctr.0020006, but it's better than
+            #skipping the data entirely AND it should also work for conforming
+            #articles.
+            for paragraph in supplementary.findall('p'):
+                suppl_div.append(paragraph)
             element_methods.remove(supplementary)
 
     def convert_verse_group_elements(self, top):
@@ -1289,7 +1423,7 @@ class OPSPLoS(OPSMeta):
                 paragraph.attrib['class'] = 'fn-type-{0}'.footnote.attrib['fn-type']
             else:
                 paragraph.attrib['class'] = 'fn'
-                #Replace the 
+                #Replace the
             element_methods.replace(footnote, paragraph)
 
     def convert_list_elements(self, top):
@@ -1386,6 +1520,7 @@ class OPSPLoS(OPSMeta):
                 if definition is None:
                     log.warning('Missing def element in def-item')
                     element_methods.remove(def_item)
+                    continue
                 #PLoS appears to consistently place all definition text in a
                 #paragraph subelement of the def element
                 def_para = definition.find('p')
@@ -1428,10 +1563,10 @@ class OPSPLoS(OPSMeta):
         """
         for graphic in top.findall('.//graphic'):
             graphic.tag = 'img'
-            ns_xlink_href = element_methods.ns_format(graphic, 'xlink:href')
             graphic.attrib['alt'] = 'unowned-graphic'
+            ns_xlink_href = element_methods.ns_format(graphic, 'xlink:href')
             if ns_xlink_href in graphic.attrib:
-                xlink_href = graphic_attributes[ns_xlink_href]
+                xlink_href = graphic.attrib[ns_xlink_href]
                 file_name = xlink_href.split('.')[-1] + '.png'
                 img_dir = 'images-' + self.doi_frag
                 img_path = '/'.join([img_dir, file_name])
@@ -1455,7 +1590,7 @@ class OPSPLoS(OPSMeta):
         #Identify subjournal name for base URl
         subjournal_name = self.doi_frag.split('.')[1]
         base_url = journal_urls[subjournal_name]
-    
+
         #Compose the address for fetchSingleRepresentation
         resource = 'fetchSingleRepresentation.action?uri=' + item_xlink_href
         return base_url.format(resource)
